/* This is the interface to the python interpreter
   Originally from the now defunct gnocci project.
   C. Steenberg 15 May 2000
*/

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <string.h>
#include <Python.h>
#include <grammar.h>
#include <node.h>
#include <parsetok.h>
#include <errcode.h>
#include <compile.h>
#include <eval.h>
#include <marshal.h>
#include "../main.h"
#include "python_int.h"


static PyObject *eval_dict,*main_o, *dict_o;
extern gint stdout_pipe[2];
gint status=0; /* 0=normal, 1=old line waiting */
extern grammar _PyParser_Grammar; /* From Python/graminit.c */
extern FILE *stdout_fp;
GString *com_buffer;
#ifndef Py_eval_input
#include <graminit.h>
#define Py_eval_input eval_input
#endif /* Py_eval_input */

int python_init(gchar *progname)
{   Py_SetProgramName(progname);
    Py_Initialize();
    PyRun_SimpleString ("import sys");
    PyRun_SimpleString ("import os");
    PyRun_SimpleString ("import Numeric");
    eval_dict = Py_BuildValue("{s{}}", "__builtins__");
    main_o=PyImport_AddModule("__main__");
    dict_o=PyModule_GetDict(main_o);
    return(1);
}

int python_singleton(SGworksheet *worksheet, gint row, gint col, PyObject *object)
{ static PyObject *s_obj;
  GtkSheet *sheet;

  sheet = GTK_SHEET(worksheet->sheet);

  s_obj=PyObject_Str(object);
  if (s_obj)
   { Py_INCREF(s_obj);
     gtk_sheet_set_cell_text(sheet,row,col,PyString_AsString (s_obj));
     Py_DECREF(s_obj);
   }
  return 1;
}

int python_sequence(SGworksheet *worksheet, gint row, gint col, 
                    PyObject *object, gint depth)
{ gint len,i;
  PyObject *seq_obj;
  GtkSheet *sheet;

  if (depth>1) return 0; /* We handle only 2D lists for now */

  sheet = GTK_SHEET(worksheet->sheet);

  len=PySequence_Length(object);

  /* Check to see if we need to resize the sheet */
  seq_obj=PySequence_GetItem(object,0);
  if (PySequence_Check(seq_obj) && gtk_sheet_get_maxcol(sheet) < len+col+1)
     sg_worksheet_add_columns(worksheet,len-gtk_sheet_get_maxcol(sheet)+col-1);
  else if (!PySequence_Check(seq_obj) && gtk_sheet_get_maxrow(sheet) < row+len)
        sg_worksheet_add_rows(worksheet,len-gtk_sheet_get_maxrow(sheet)+row-1);

  for (i=0;i<len;i++)
    { seq_obj=PySequence_GetItem(object,i);
      Py_INCREF(seq_obj);
      if (PySequence_Check(seq_obj))
        python_sequence(worksheet,row,col+i,seq_obj,depth+1);
      else
        python_singleton(worksheet,row+i,col,seq_obj);
      Py_XDECREF(seq_obj);
    }
  return 1;
}


int python_sheet(SGworksheet *worksheet, gint row, gint col, gchar *command)
{ PyObject *object=NULL;
  gchar errmsg[]="Error!",*msg;
  GtkSheet *sheet;

  sheet = GTK_SHEET(worksheet->sheet);

  object=PyRun_String (command, Py_eval_input, dict_o, dict_o);

  if (!object)
   { if (PyErr_Occurred()) PyErr_Clear();
     PyRun_String (command, Py_single_input, dict_o, dict_o);
   }

  if (object)
   {  Py_INCREF(object);
      if (PySequence_Check(object))
        python_sequence(worksheet,row,col,object,0);
      else
        python_singleton(worksheet,row,col,object);
      
      Py_XDECREF(object);
   }
  else
   { if (PyErr_Occurred())
      { msg=errmsg;
        PyErr_Clear();
      }
     else msg=command;
     gtk_sheet_set_cell_text(GTK_SHEET(sheet),row,col,msg);
     return 0;
   }
  return 1;
}
