/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include <gdk/gdk.h>
#include "main.h"
#include "../pixmaps/sg.xpm"
#include <Python.h>
#include "python/python_int.h"


gint
quit(GtkWidget *widget)
{
  gint veto;

  sg_dialogs_destroy();

  veto = sg_accept_dialog("Exit SciGraphica?");
  if(!veto) return TRUE;

  veto = sg_accept_dialog("Save project before leaving?");
  if(veto) sg_project_export();

  return FALSE;
}

gint
main_quit (GtkWidget *widget)
{
  sg_project_close();

  gtk_main_quit();

  return FALSE;
}

int main(int argc, char *argv[]){
 GtkWidget *vbox1;
 GtkWidget *hbox1;
 GtkWidget *label1;
 GtkWidget *label2;
 GdkPixmap *pixmap;
 GdkBitmap *mask;

 gtk_init(&argc,&argv);
 python_init(argv[0]);

 /* Initialize global variables */

 use_custom_delimiter = SG_DELIMITER_UNKNOWN;
 delimiter = SG_DELIMITER_UNKNOWN;
 custom_delimiter = NULL;
 skip_lines = 0;
 rename_worksheets = TRUE;
 rename_columns = FALSE;

 last_project_path = g_strdup(g_get_current_dir());
 last_project_filename = g_strdup("unknown.sg");
 last_plot_path = g_strdup(g_get_current_dir());
 last_plot_filename = g_strdup("unknown");
 last_worksheet_path = g_strdup(g_get_current_dir());
 last_worksheet_filename = g_strdup("unknown");

 /* Build GUI */

 main_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
 gtk_window_set_title(GTK_WINDOW(main_window), "SciGraphica");
 gtk_widget_set_usize(main_window, 400, 400);
 gtk_container_border_width(GTK_CONTAINER(main_window),0);

 gtk_widget_show(main_window);

 pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                gdk_colormap_get_system(),
                                                &mask, NULL, sg_xpm);
                          
 gdk_window_set_icon(main_window->window, NULL, pixmap, mask);            

 sg_presentation_dialog();

 gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
		     GTK_SIGNAL_FUNC (main_quit), NULL);

 vbox1=gtk_vbox_new(FALSE,0);
 gtk_container_add(GTK_CONTAINER(main_window),vbox1);
 gtk_widget_show(vbox1);

 sg_project_build_menu();
 gtk_box_pack_start(GTK_BOX(vbox1), menu_bar, FALSE, FALSE, 2);
 gtk_widget_show(menu_bar);

 sg_project_build_popups();
 sg_worksheet_build_menu();
 sg_plot_build_menu();

 hbox1=gtk_hbox_new(FALSE,0);
 gtk_container_set_border_width(GTK_CONTAINER(hbox1), 5);
 gtk_box_set_spacing(GTK_BOX(hbox1), 5);
 gtk_box_pack_start(GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);
 gtk_widget_show(hbox1);

 gui_notebook = gtk_notebook_new();
 gtk_box_pack_start(GTK_BOX(hbox1),gui_notebook, TRUE, TRUE,0);
 gtk_widget_show(gui_notebook);

 scrollw1=gtk_scrolled_window_new(NULL, NULL);
 gtk_container_border_width(GTK_CONTAINER(scrollw1),0);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw1),
				GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 label1 = gtk_label_new("Worksheets");
 gtk_notebook_append_page(GTK_NOTEBOOK(gui_notebook), scrollw1, label1);
 gtk_widget_show(scrollw1);

 scrollw2=gtk_scrolled_window_new(NULL, NULL);
 gtk_container_border_width(GTK_CONTAINER(scrollw2),0);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw2),
				GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 label2 = gtk_label_new("Plots");
 gtk_notebook_append_page(GTK_NOTEBOOK(gui_notebook), scrollw2, label2);
 gtk_widget_show(scrollw2);


 iconlist1 = gtk_icon_list_new(48, 1);
 gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw1),iconlist1);
 gtk_signal_connect(GTK_OBJECT(iconlist1),"select_icon",
                    (GtkSignalFunc) sg_project_open_worksheet, NULL);

 iconlist2 = gtk_icon_list_new(48, 1);
 gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw2),iconlist2);
 gtk_signal_connect(GTK_OBJECT(iconlist2),"select_icon",
                    (GtkSignalFunc) sg_project_open_plot, NULL);

 gtk_widget_show(iconlist1);
 gtk_widget_show(iconlist2);

 gtk_widget_show(vbox1);
 
 gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
                    (GtkSignalFunc)quit, NULL);

 sg_project_new();
 sg_project_new_worksheet();

 gtk_main();

 gdk_pixmap_unref(pixmap);
 gdk_bitmap_unref(mask);

 g_free(last_plot_path);
 g_free(last_project_path);
 g_free(last_worksheet_path);
 g_free(last_plot_filename);
 g_free(last_project_filename);
 g_free(last_worksheet_filename);

 return (0);
}


