/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

SGdataset *
sg_dataset_new (void)
{
  SGdataset *dataset;

  dataset = g_new(SGdataset, 1);

  dataset->real_data = gtk_plot_data_new(); 

  dataset->worksheet = NULL;

  dataset->x = -1;
  dataset->y = -1;
  dataset->dx = -1;
  dataset->dy = -1;
  dataset->l = -1;

  return dataset;
}

SGdataset *
sg_dataset_new_with_worksheet  (SGworksheet *worksheet,
				gint col_x,
				gint col_y,
				gint col_ex,
				gint col_ey,
				gint col_l)
{
  SGdataset *dataset;

  dataset = sg_dataset_new();
  sg_dataset_set_worksheet(dataset, worksheet, 
                           col_x, col_y, col_ex, col_ey, col_l);

  return dataset;
}

void
sg_dataset_set_worksheet  (SGdataset *dataset,
			   SGworksheet *worksheet,
		   	   gint col_x,
			   gint col_y,
			   gint col_ex,
			   gint col_ey,
			   gint col_l)
{
  dataset->worksheet = worksheet;

  dataset->x = col_x;
  dataset->y = col_y;
  dataset->dx = col_ex;
  dataset->dy = col_ey;
  dataset->l = col_l;

  sg_dataset_refresh(dataset);
}


void
sg_dataset_add_point(SGdataset *dataset,
                     gdouble *x, gdouble *y, 
                     gdouble *ex, gdouble *ey,
                     gchar *label)
{
  gint num_points;

  num_points = dataset->real_data->num_points + 1;

  dataset->real_data->x = (gdouble *)g_realloc(dataset->real_data->x,
                                               num_points * sizeof(double));
  dataset->real_data->y = (gdouble *)g_realloc(dataset->real_data->y,
                                               num_points * sizeof(double));
  dataset->real_data->dx = (gdouble *)g_realloc(dataset->real_data->dx,
                                                num_points * sizeof(double));
  dataset->real_data->dy = (gdouble *)g_realloc(dataset->real_data->dy,
                                                num_points * sizeof(double));
  dataset->real_data->labels = (gchar **)g_realloc(dataset->real_data->labels,
                                                  num_points * sizeof(gchar *));

  dataset->real_data->x[num_points-1] = *x; 
  dataset->real_data->y[num_points-1] = *y; 
  dataset->real_data->dx[num_points-1] = *ex; 
  dataset->real_data->dy[num_points-1] = *ey; 
  dataset->real_data->labels[num_points-1] = NULL;

  if(label) 
    dataset->real_data->labels[num_points-1] = g_strdup(label);
  
  dataset->real_data->num_points++;

}

void
sg_dataset_refresh(SGdataset *dataset)
{
  SGworksheet *worksheet;
  GtkSheet *sheet;
  gdouble *x = NULL, *y = NULL, *ex = NULL, *ey = NULL;
  gchar **labels = NULL;
  gint row, col_x, col_y, col_ex, col_ey, col_l;
  gchar *text[5];
  gchar name[100];
  gchar x_name[100];
  gchar y_name[100];
  gint numrows = 0;
  gint begin;
  gint end;
  gint n;

  sg_dataset_free_points(dataset);

  worksheet = dataset->worksheet;
  sheet = GTK_SHEET(worksheet->sheet);

  col_x = dataset->x;
  col_y = dataset->y;
  col_ex = dataset->dx;
  col_ey = dataset->dy;
  col_l = dataset->l;

  begin = MAX(0, worksheet->begin);
  end = sheet->maxallocrow;
 
  if(worksheet->end >= 0)
    end = MIN(end, worksheet->end);

  if(worksheet->begin <= sheet->maxallocrow){ 
    n = end - begin + 1;
    x = (gdouble *)g_malloc(n*sizeof(gdouble));
    y = (gdouble *)g_malloc(n*sizeof(gdouble));
    ex = (gdouble *)g_malloc(n*sizeof(gdouble));
    ey = (gdouble *)g_malloc(n*sizeof(gdouble));
    labels = (gchar **)g_malloc(n*sizeof(gchar *));

    for(row = begin; row <= end; row++){
      text[0] = gtk_sheet_cell_get_text(sheet, row, col_x);      
      text[1] = gtk_sheet_cell_get_text(sheet, row, col_y);      
      if(col_ex >= 0)
          text[2] = gtk_sheet_cell_get_text(sheet, row, col_ex);      
      if(col_ey >= 0)
          text[3] = gtk_sheet_cell_get_text(sheet, row, col_ey);      
      if(col_l >= 0)
          text[4] = gtk_sheet_cell_get_text(sheet, row, col_l);      

      if(text[0] && strlen(text[0]) > 0 &&
         text[1] && strlen(text[1]) > 0){
            x[numrows] = atof((text[0] == NULL ? "0" : text[0]));      
            y[numrows] = atof((text[1] == NULL ? "0" : text[1]));      
            if(col_ex >= 0)
               ex[numrows] = atof((text[2] == NULL ? "0" : text[2]));      
            else
               ex[numrows] = 0.;
            if(col_ex >= 0)
               ey[numrows] = atof((text[3] == NULL ? "0" : text[3]));      
            else
               ey[numrows] = 0.;
            if(col_l >= 0 && text[4] != NULL)
               labels[numrows] = g_strdup(text[4]);      
            else
               labels[numrows] = NULL;

            numrows++;
      }
    }

    gtk_plot_data_set_points(dataset->real_data, x, y, ex, ey, numrows);  
    gtk_plot_data_set_labels(dataset->real_data, labels);  

  }
  
  if(!GTK_SHEET(worksheet->sheet)->column[col_x].name)
     sprintf(x_name,"%d",col_x);
  else
     sprintf(x_name,"%s",GTK_SHEET(worksheet->sheet)->column[col_x].name);

  if(!GTK_SHEET(worksheet->sheet)->column[col_y].name)
     sprintf(y_name,"%d",col_y);
  else
     sprintf(y_name,"%s",GTK_SHEET(worksheet->sheet)->column[col_y].name);


  sprintf(name,"%s->%s(X),%s(Y)",sheet->name, x_name, y_name);
  sg_dataset_set_name(dataset, name);
}

void
sg_dataset_destroy(SGdataset *dataset)
{
  sg_dataset_free_points(dataset);

  if(dataset->real_data->name) g_free(dataset->real_data->name);
  if(dataset->real_data->legend) g_free(dataset->real_data->legend);

  g_free(dataset);
}

void
sg_dataset_free_points(SGdataset* dataset)
{
  if(dataset->real_data->x) g_free(dataset->real_data->x);
  if(dataset->real_data->y) g_free(dataset->real_data->y);
  if(dataset->real_data->dx) g_free(dataset->real_data->dx);
  if(dataset->real_data->dy) g_free(dataset->real_data->dy);
  if(dataset->real_data->labels) {
      gint i;
      for (i = 0; i < dataset->real_data->num_points; i++){
            g_free(dataset->real_data->labels[i]);
      }
      g_free(dataset->real_data->labels);
  }

  dataset->real_data->x = NULL;
  dataset->real_data->y = NULL;
  dataset->real_data->dx = NULL;
  dataset->real_data->dy = NULL;
  dataset->real_data->labels = NULL;
}


void
sg_dataset_set_name(SGdataset *dataset, gchar *name)
{
  gtk_plot_data_set_name(dataset->real_data, name);
}
                     
