/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

#define DATASET_NAME_LENGTH 100

static GtkWidget *worksheet_combo;
static GtkWidget *datasets_list, *columns_list;
static GtkWidget *x_entry, *y_entry, *dx_entry, *dy_entry, *l_entry; 
static SGplot *plot;
GList *columns = NULL;
GList *old_datasets = NULL;

static SGdataset *find_dataset				(gint row, 
							 gint *col_x, 
							 gint *col_y, 
							 gint *col_ex, 
							 gint *col_ey,
							 gint *col_l);

static gchar *titles[] = {"          Worksheet          ",
                          "     X     ",
                          "     Y     ",
                          "    Xerr   ",
                          "    Yerr   ", 
                          "    Lbls   ", 
                          NULL};

void 
sg_data_dialog_update_plot(GtkWidget *widget)
{
  GList *list = NULL, *aux_list = NULL;
  SGworksheet *worksheet;
  SGdataset *dataset;
  SGdataset *aux_data;
  GtkSheet *sheet;
  gint i, col;
  gint col_x = -1, col_y = -1, col_ex = -1, col_ey = -1, col_l = -1;
  gchar *text[6], *title; 
  gchar default_title[10];

  old_datasets = active_layer->datasets;

  active_layer->datasets = NULL;

  /* REMOVE OLD DATASETS */

  aux_list = GTK_PLOT(active_layer->real_plot)->data_sets;
  while(aux_list){
    gtk_plot_remove_data(GTK_PLOT(active_layer->real_plot), 
                         (GtkPlotData *)aux_list->data);
    aux_list = GTK_PLOT(active_layer->real_plot)->data_sets;
  }
  GTK_PLOT(active_layer->real_plot)->data_sets = NULL;

  /* ADD NEW DATASETS */

  for(i = 0; i < GTK_CLIST(datasets_list)->rows; i++){
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 0, &text[0]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 1, &text[1]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 2, &text[2]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 3, &text[3]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 4, &text[4]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 5, &text[5]);

    list = worksheets;
    while(list){
      worksheet = (SGworksheet *)list->data;
      sheet = GTK_SHEET(worksheet->sheet);
      if(strcmp(worksheet->name, text[0]) == 0){
        col_ex = -1;
        col_ey = -1;
        for(col = 0; col <= sheet->maxcol; col++){
           title = sheet->column[col].name;
           if(!title){
             sprintf(default_title,"%d",col);
             title = default_title;
           }
           if(strcmp(text[1], title) == 0) col_x = col;
           if(strcmp(text[2], title) == 0) col_y = col;
           if(text[3] && strcmp(text[3], title) == 0) col_ex = col;
           if(text[4] && strcmp(text[4], title) == 0) col_ey = col;
           if(text[5] && strcmp(text[5], title) == 0) col_l = col;
        }

        aux_list = old_datasets;
        while(aux_list){
          gboolean found;

          aux_data = (SGdataset *)aux_list->data;

          found = (col_x == aux_data->x &&
                   col_y == aux_data->y);

          if(found) break;

          aux_list = aux_list->next;
        }
       
        if(aux_list){
            dataset = aux_data;
            old_datasets = g_list_remove_link(old_datasets, aux_list);
            g_list_free_1(aux_list);
        } else { 
            dataset = sg_dataset_new_with_worksheet(worksheet, 
                                                    col_x, col_y, 
                                                    col_ex, col_ey, col_l);
        }

        sg_layer_add_dataset(active_layer, dataset);
      }
      list = list->next;
    }
  }

  aux_list = old_datasets;
  while(aux_list){
     aux_data = (SGdataset *)aux_list->data;

     sg_dataset_destroy(aux_data);
     old_datasets = g_list_remove_link(old_datasets, aux_list);
     g_list_free_1(aux_list);
     aux_list = old_datasets;
  }

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static gint
get_column_from_name(gchar *name)
{
  GList *list;
  gint n = 0;
 
  list = columns;
  while(list){
    if(strcmp(name, (gchar *)list->data) == 0) return n;
    n++;
    list = list->next;
  }
 
  return -1;
}

static void
update_datasets(GtkEntry *entry)
{
  gint i, nrows;
  SGworksheet *worksheet = NULL;
  GList *list;
  gchar name[DATASET_NAME_LENGTH];
  gchar *text;

  nrows = GTK_CLIST(columns_list)->rows;
  for(i = 0; i <= nrows; i++){
    gtk_clist_remove(GTK_CLIST(columns_list), 0);
  }
  
  text = gtk_entry_get_text(entry);

  list = worksheets;
  while(list){
    worksheet = (SGworksheet *)list->data;
    if(strcmp(text, worksheet->name) == 0) break;
    list = list->next;
  } 

  for(i=0; i <= GTK_SHEET(worksheet->sheet)->maxcol; i++){
     if(!GTK_SHEET(worksheet->sheet)->column[i].name)
        sprintf(name,"%d",i);
     else
        sprintf(name,"%s",GTK_SHEET(worksheet->sheet)->column[i].name);
     text=g_strdup(name);
     gtk_clist_append(GTK_CLIST(columns_list), &text);
     columns = g_list_append(columns, text);
  };

  gtk_entry_set_text(GTK_ENTRY(x_entry), "");
  gtk_entry_set_text(GTK_ENTRY(y_entry), "");
  gtk_entry_set_text(GTK_ENTRY(dx_entry), "");
  gtk_entry_set_text(GTK_ENTRY(dy_entry), "");
  gtk_entry_set_text(GTK_ENTRY(l_entry), "");
}

static gint
set_column(GtkWidget *widget, gpointer data)
{
  GList *selection;
  gchar *text;
  gint row;
 
  selection = GTK_CLIST(columns_list)->selection;
  if(!selection) return TRUE;
 
  text = gtk_entry_get_text(GTK_ENTRY(data));
  if(strlen(text) > 0) return TRUE;

  row = GPOINTER_TO_INT(selection->data);
  gtk_clist_get_text(GTK_CLIST(columns_list), row, 0, &text);
 
  gtk_entry_set_text(GTK_ENTRY(data), text);
  gtk_clist_remove(GTK_CLIST(columns_list), row);
  return TRUE;
}

static gint
restore_column(GtkWidget *widget, gpointer data)
{
  gchar *text;
  gint col;

  text = gtk_entry_get_text(GTK_ENTRY(data));
  if(strlen(text) == 0) return TRUE;

  col = get_column_from_name(text);
  gtk_clist_insert(GTK_CLIST(columns_list), col, &text);

  gtk_entry_set_text(GTK_ENTRY(data), "");
  return TRUE;
}

static gint
add_dataset(GtkWidget *widget, gpointer data)
{
  gchar *text[6];
  gchar *dtext[6];
  gint i;

  text[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(worksheet_combo)->entry));
  text[1] = gtk_entry_get_text(GTK_ENTRY(x_entry));
  text[2] = gtk_entry_get_text(GTK_ENTRY(y_entry));
  text[3] = gtk_entry_get_text(GTK_ENTRY(dx_entry));
  text[4] = gtk_entry_get_text(GTK_ENTRY(dy_entry));
  text[5] = gtk_entry_get_text(GTK_ENTRY(l_entry));

  if(strlen(text[0]) == 0 || strlen(text[1]) == 0) return TRUE;

  for(i = 0; i < GTK_CLIST(datasets_list)->rows; i++){
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 0, &dtext[0]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 1, &dtext[1]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 2, &dtext[2]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 3, &dtext[3]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 4, &dtext[4]);
    gtk_clist_get_text(GTK_CLIST(datasets_list), i, 5, &dtext[5]);

    if(strcmp(text[0], dtext[0]) == 0 &&
       strcmp(text[1], dtext[1]) == 0 &&
       strcmp(text[2], dtext[2]) == 0){
         
       if(text[3] && strlen(text[3]) > 0) 
         gtk_clist_set_text(GTK_CLIST(datasets_list), i, 3, text[3]);
       if(text[4] && strlen(text[4]) > 0) 
         gtk_clist_set_text(GTK_CLIST(datasets_list), i, 4, text[4]);
       if(text[5] && strlen(text[5]) > 0) 
         gtk_clist_set_text(GTK_CLIST(datasets_list), i, 5, text[5]);
       break;
    }
  }

  if(i == GTK_CLIST(datasets_list)->rows)
             gtk_clist_append(GTK_CLIST(datasets_list), text);
 
  update_datasets(GTK_ENTRY(GTK_COMBO(worksheet_combo)->entry));
  return TRUE;
}

static gint
remove_dataset(GtkWidget *widget, gpointer data)
{
  GList *selection;
  gint row;

  selection = GTK_CLIST(datasets_list)->selection;
  if(!selection) return TRUE;

  row = GPOINTER_TO_INT(selection->data);
  gtk_clist_remove(GTK_CLIST(datasets_list), row);

  return TRUE;
}

static gint
refresh_dataset(GtkWidget *widget, gpointer data)
{
  SGdataset *aux_dataset;
  GList *selection;
  gint row;
  gint col_x, col_y, col_ex, col_ey, col_l;

  selection = GTK_CLIST(datasets_list)->selection;
  if(!selection) return TRUE;

  row = GPOINTER_TO_INT(selection->data);

  aux_dataset = find_dataset(row, &col_x, &col_y, &col_ex, &col_ey, &col_l);
  if(aux_dataset){
     aux_dataset->dx = col_ex;
     aux_dataset->dy = col_ey;
     aux_dataset->l = col_l;
     sg_dataset_refresh(aux_dataset);
  }

  return TRUE; 
}

static SGdataset * 
find_dataset(gint row, 
             gint *col_x, gint *col_y, gint *col_ex, gint *col_ey, gint *col_l)
{
  SGworksheet *worksheet;
  GtkSheet *sheet;
  GList *list, *old_datasets = NULL;
  gchar *text[6], *title, default_title[100];
  gint col;

  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 0, &text[0]);
  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 1, &text[1]);
  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 2, &text[2]);
  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 3, &text[3]);
  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 4, &text[4]);
  gtk_clist_get_text(GTK_CLIST(datasets_list), row, 5, &text[5]);

  list = worksheets;
  while(list){
    worksheet = (SGworksheet *)list->data;
    sheet = GTK_SHEET(worksheet->sheet);
    if(strcmp(worksheet->name, text[0]) == 0){
      *col_ex = -1;
      *col_ey = -1;
      *col_l = -1;
      for(col = 0; col <= sheet->maxcol; col++){
           title = sheet->column[col].name;
           if(!title){
             sprintf(default_title,"%d",col);
             title = default_title;
           }
           if(strcmp(text[1], title) == 0) *col_x = col;
           if(strcmp(text[2], title) == 0) *col_y = col;
           if(strcmp(text[3], title) == 0) *col_ex = col;
           if(strcmp(text[4], title) == 0) *col_ey = col;
           if(strcmp(text[5], title) == 0) *col_l = col;
      }

      old_datasets = active_layer->datasets;
      while(old_datasets){
        SGdataset *aux_data;

        aux_data = (SGdataset *)old_datasets->data;

        if (*col_x == aux_data->x && *col_y == aux_data->y){
              return (SGdataset *)old_datasets->data;
        }

        old_datasets = old_datasets->next;
      }

      break;
    }

    list = list->next;
  }
 
  return NULL;
}


GtkWidget *
sg_data_dialog_new (SGlayer *layer)
{
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *main_box;
  GtkWidget *table;
  GtkWidget *box, *sw;
  GtkWidget *action_area;
  GtkWidget *x1_button, *x2_button;
  GtkWidget *y1_button, *y2_button;
  GtkWidget *dx1_button, *dx2_button;
  GtkWidget *dy1_button, *dy2_button;
  GtkWidget *l1_button, *l2_button;
  GtkWidget *add_button, *remove_button, *refresh_button;
  GList *list;
  GtkWidget *item;
  GtkPlot *p;
  SGworksheet *worksheet;
  SGdataset *dataset;
  gchar *text[6];
  gchar *default_text = "";


  p = GTK_PLOT(layer->real_plot);
  plot = (SGplot *)layer->parent;
  active_layer = layer;

  /* Create widgets */
  main_box = gtk_vbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 10);

  box = gtk_hbox_new(TRUE, 5);
  label = gtk_label_new("Select Worksheet:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_box_pack_start (GTK_BOX (main_box), box, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
  worksheet_combo = gtk_combo_new();
  gtk_box_pack_end (GTK_BOX (box), worksheet_combo, FALSE, FALSE, 0);
  gtk_widget_show(box);

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(worksheet_combo)->entry), FALSE);
  list = worksheets;
  while(list){
     worksheet = (SGworksheet *)list->data;
     item = gtk_list_item_new_with_label(worksheet->name);
     gtk_widget_show(item);
     gtk_container_add(GTK_CONTAINER(GTK_COMBO(worksheet_combo)->list), item);
     list = list->next;
  }

  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(worksheet_combo)->entry), 
                     "changed",
                     (GtkSignalFunc) update_datasets, NULL);

  frame = gtk_frame_new("Columns");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  table = gtk_table_new(5, 5, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_container_add (GTK_CONTAINER (frame), table);

  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_usize(sw, 160, 160);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE (table), sw, 0, 1, 0, 5);

  columns_list = gtk_clist_new(1);
  gtk_container_add(GTK_CONTAINER(sw), columns_list);

/********************************************************************/
  x1_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(x1_button), 
                    gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), x1_button, 1, 2, 0, 1);

  x2_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(x2_button),
                    gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), x2_button, 2, 3, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(" X: "), 3, 4, 0, 1 );
  x_entry = gtk_entry_new();
  gtk_entry_set_editable(GTK_ENTRY(x_entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), x_entry, 4, 5, 0, 1 );
  gtk_signal_connect(GTK_OBJECT(x1_button), "clicked",
                     (GtkSignalFunc) restore_column, x_entry);
  gtk_signal_connect(GTK_OBJECT(x2_button), "clicked",
                     (GtkSignalFunc) set_column, x_entry);

/********************************************************************/
  y1_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(y1_button), 
                    gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), y1_button, 1, 2, 1, 2);

  y2_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(y2_button),
                    gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), y2_button, 2, 3, 1, 2);

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(" Y: "), 3, 4, 1, 2 );
  y_entry = gtk_entry_new();
  gtk_entry_set_editable(GTK_ENTRY(y_entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), y_entry, 4, 5, 1, 2 );
  gtk_signal_connect(GTK_OBJECT(y1_button), "clicked",
                     (GtkSignalFunc) restore_column, y_entry);
  gtk_signal_connect(GTK_OBJECT(y2_button), "clicked",
                     (GtkSignalFunc) set_column, y_entry);


/********************************************************************/
  dx1_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(dx1_button), 
                    gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), dx1_button, 1, 2, 2, 3);

  dx2_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(dx2_button),
                    gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), dx2_button, 2, 3, 2, 3);

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(" Xerr: "), 3, 4, 2, 3 );
  dx_entry = gtk_entry_new();
  gtk_entry_set_editable(GTK_ENTRY(dx_entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), dx_entry, 4, 5, 2, 3 );
  gtk_signal_connect(GTK_OBJECT(dx1_button), "clicked",
                     (GtkSignalFunc) restore_column, dx_entry);
  gtk_signal_connect(GTK_OBJECT(dx2_button), "clicked",
                     (GtkSignalFunc) set_column, dx_entry);


/********************************************************************/
  dy1_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(dy1_button), 
                    gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), dy1_button, 1, 2, 3, 4);

  dy2_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(dy2_button),
                    gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), dy2_button, 2, 3, 3, 4);

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(" Yerr: "), 3, 4, 3, 4 );
  dy_entry = gtk_entry_new();
  gtk_entry_set_editable(GTK_ENTRY(dy_entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), dy_entry, 4, 5, 3, 4 );
  gtk_signal_connect(GTK_OBJECT(dy2_button), "clicked",
                     (GtkSignalFunc) set_column, dy_entry);
  gtk_signal_connect(GTK_OBJECT(dy1_button), "clicked",
                     (GtkSignalFunc) restore_column, dy_entry);

/********************************************************************/
  l1_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(l1_button), 
                    gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), l1_button, 1, 2, 4, 5);

  l2_button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(l2_button),
                    gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT));
  gtk_table_attach_defaults(GTK_TABLE (table), l2_button, 2, 3, 4, 5);

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(" Lbls: "), 3, 4, 4, 5);
  l_entry = gtk_entry_new();
  gtk_entry_set_editable(GTK_ENTRY(l_entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), l_entry, 4, 5, 4, 5 );
  gtk_signal_connect(GTK_OBJECT(l2_button), "clicked",
                     (GtkSignalFunc) set_column, l_entry);
  gtk_signal_connect(GTK_OBJECT(l1_button), "clicked",
                     (GtkSignalFunc) restore_column, l_entry);

/********************************************************************/
  action_area = gtk_hbutton_box_new ();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area), GTK_BUTTONBOX_START);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(action_area), 5);
  gtk_box_pack_start (GTK_BOX (main_box), action_area, FALSE, FALSE, 0);

  add_button = gtk_button_new_with_label ("Add");
  gtk_box_pack_start (GTK_BOX (action_area), add_button, TRUE, TRUE, 0);
  gtk_widget_show (add_button);

  remove_button = gtk_button_new_with_label ("Remove");
  gtk_box_pack_start (GTK_BOX (action_area), remove_button, TRUE, TRUE, 0);
  gtk_widget_show (remove_button);

  refresh_button = gtk_button_new_with_label ("Refresh");
  gtk_box_pack_start (GTK_BOX (action_area), refresh_button, TRUE, TRUE, 0);
  gtk_widget_show (refresh_button);


/********************************************************************/
  frame = gtk_frame_new("Datasets");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
  gtk_widget_set_usize(sw, 320, 160);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(frame), sw);
  datasets_list = gtk_clist_new_with_titles(6, titles);
  gtk_container_add(GTK_CONTAINER(sw), datasets_list);

  /* connect signals */
  gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
                     (GtkSignalFunc) add_dataset, NULL);

  gtk_signal_connect(GTK_OBJECT(remove_button), "clicked",
                     (GtkSignalFunc) remove_dataset, NULL);

  gtk_signal_connect(GTK_OBJECT(refresh_button), "clicked",
                     (GtkSignalFunc) refresh_dataset, NULL);

  /* Show widgets */
  list = layer->datasets;
  while(list){
    GtkSheet *sheet;

    dataset = (SGdataset *)list->data;
    sheet = GTK_SHEET(dataset->worksheet->sheet);

    text[0] = dataset->worksheet->name;
    text[1] = sheet->column[dataset->x].name;
    text[2] = sheet->column[dataset->y].name;
    text[3] = default_text;
    text[4] = default_text;
    text[5] = default_text;
    if(dataset->dx >= 0)
      text[3] = sheet->column[dataset->dx].name;
    if(dataset->dy >= 0)
      text[4] = sheet->column[dataset->dy].name;
    if(dataset->l >= 0)
      text[5] = sheet->column[dataset->l].name;

    gtk_clist_append(GTK_CLIST(datasets_list), text);

    list = list->next;
  }
 
  update_datasets(GTK_ENTRY(GTK_COMBO(worksheet_combo)->entry));

  return main_box;
}

