/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>

#include "main.h"

static FILE *fp;

void
sg_worksheet_file_import_ascii(SGworksheet *worksheet, gchar *file)
{
  GtkSheet *sheet;
  gint row, col;
  gint nchar, npos;
  gboolean newcol = TRUE;
  gboolean space = FALSE;
  gchar c;
  gchar *line;
  gint nlines = 0;
  gchar label[255];

  sheet = GTK_SHEET(worksheet->sheet);
  row = 0;
  col = 0;
  nchar = 0;
  npos = 0;

  if((fp = fopen(file, "r")) == NULL){
    g_warning("ERROR: Cannot open file: %s", file);
    return;
  }

  gtk_sheet_freeze(sheet);

  line = (gchar *)g_malloc(sizeof(gchar));

  while((c = fgetc(fp)) != -1){
    switch (c){
     case ',':
       if(use_custom_delimiter) break;
       if(delimiter != SG_DELIMITER_UNKNOWN && delimiter != SG_DELIMITER_COMMA) break;
       if(nlines < skip_lines) break;
       newcol = TRUE;
       if(newcol && space){
         space = FALSE;
         break;
       }
       space = FALSE;
       line[npos]='\0';
       if(nlines == 0 && rename_columns){
           gtk_sheet_column_button_add_label(sheet, col, line);
           gtk_sheet_set_column_title(sheet, col, line);
       } else {
           gtk_sheet_set_cell_text(sheet, row, col, line);
       }
       npos = 0;
       col++;
       if(col > sheet->maxcol)
         sg_worksheet_add_columns(worksheet, 1);
       break;
     case '\0':
     case '\n':
       if(nlines < skip_lines) break;
       if(newcol) break;
       newcol = TRUE;
       space = FALSE;
       line[npos]='\0';
       if(nlines == 0 && rename_columns){
           gtk_sheet_column_button_add_label(sheet, col, line);
           gtk_sheet_set_column_title(sheet, col, line);
           row--;
       } else {
           gtk_sheet_set_cell_text(sheet, row, col, line);
       }
       sprintf(label, "Row %d\n", row);
       gtk_label_set(GTK_LABEL(worksheet->label), label);
       npos = 0;
       row ++;
       if(row > sheet->maxrow)
         sg_worksheet_add_rows(worksheet, 1);
       col = 0;
       nlines++;
       sprintf(label, "READING ROW : %d",nlines);
       gtk_label_set_text(GTK_LABEL(worksheet->label), label);
       break;
     case '\t':
     case ' ':
       if(nlines < skip_lines) break;
       if(delimiter != SG_DELIMITER_UNKNOWN && delimiter != SG_DELIMITER_TAB) break;
       if(newcol || space) break;
       newcol = TRUE;
       space = TRUE;
       line[npos]='\0';
       if(nlines == 0 && rename_columns){
           gtk_sheet_column_button_add_label(sheet, col, line);
           gtk_sheet_set_column_title(sheet, col, line);
       } else {
           gtk_sheet_set_cell_text(sheet, row, col, line);
       }
       npos = 0;
       col ++;
       if(col > sheet->maxcol)
         sg_worksheet_add_columns(worksheet, 1);
       break;       
     default:
       if(!use_custom_delimiter){ 
         newcol = FALSE;
         space = FALSE;
         line = (gchar *)g_realloc(line, (npos+2)*sizeof(gchar));
         line[npos++] = c;
         break;
       }
       if(newcol || space) break;
       if(custom_delimiter != NULL && strlen(custom_delimiter) > 0 &&
          strcmp(&c, custom_delimiter) == 0){  
         newcol = TRUE;
         space = TRUE;
         line[npos]='\0';
         if(nlines == 0 && rename_columns){
             gtk_sheet_column_button_add_label(sheet, col, line);
             gtk_sheet_set_column_title(sheet, col, line);
         } else {
             gtk_sheet_set_cell_text(sheet, row, col, line);
         }
         npos = 0;
         col ++;
         if(col > sheet->maxcol)
           sg_worksheet_add_columns(worksheet, 1);
       }
    }
  }

  g_free(line);
  sprintf(label, "%s:%d",sheet->column[sheet->active_cell.col].name,
                         sheet->active_cell.row);
  gtk_label_set(GTK_LABEL(worksheet->label), label);
  gtk_sheet_thaw(sheet);
}

gchar *
xml_get_string(gchar *input)
{
  gchar *c;
  gchar *output = NULL;
  gint n = 1;

  output = (gchar *)g_malloc(sizeof(gchar));

  c = input;
  while(c && *c != '\0'){
    switch(*c){
        case '>':
          output = (gchar *)g_realloc(output, (n + 3) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'g';
          output [n + 1] = 't';
          output [n + 2] = ';';
          n += 4;
          break;
        case '<':
          output = (gchar *)g_realloc(output, (n + 3) * sizeof(gchar));
          output [n - 1] = '&';

          output [n] = 'l';
          output [n + 1] = 't';
          output [n + 2] = ';';
          n += 4;
          break;
        case '&':
          output = (gchar *)g_realloc(output, (n + 4) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'a';
          output [n + 1] = 'm';
          output [n + 2] = 'p';
          output [n + 3] = ';';
          n += 5;
          break;
        default:
          output = (gchar *)g_realloc(output, n * sizeof(gchar));
          output[n-1] = *c;
          n += 1;
          break;
    }
    c++;
  }


  output = (gchar *)g_realloc(output, n * sizeof(gchar));
  output[n-1] = '\0';

  return output;
}


