/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

static GtkWidget *delimiter_item;
static GtkWidget *delimiter_combo;
static GtkWidget *delimiter_entry;
static GtkWidget *skip_spin;
static GtkWidget *rename_columns_item;
static GtkWidget *rename_worksheets_item;

static gchar *delimiters[] = {
                              "unknown",
                              "comma(,)",
                              "tab",
                              NULL};
                              
static gboolean
mw_destroy(GtkWidget *widget)
{
  /* This is needed to get out of gtk_main */
  gtk_main_quit ();

  return FALSE;
}

static void
update_options(GtkWidget *widget, gpointer data)
{
  GtkWidget *child;

  use_custom_delimiter = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(delimiter_item));

  child = GTK_LIST(GTK_COMBO(delimiter_combo)->list)->selection->data;
  delimiter = gtk_list_child_position(GTK_LIST(GTK_COMBO(delimiter_combo)->list), child);

  custom_delimiter = gtk_entry_get_text(GTK_ENTRY(delimiter_entry)); 

  skip_lines = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(skip_spin));

  rename_columns = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rename_columns_item));

  rename_worksheets = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rename_worksheets_item));

  gtk_widget_destroy(widget);
}

static void
init_dialog()
{
  sg_combo_set_items(GTK_COMBO(delimiter_combo), delimiters);
    
  gtk_list_select_item(GTK_LIST(GTK_COMBO(delimiter_combo)->list), delimiter);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(skip_spin), skip_lines);

  if(custom_delimiter)
           gtk_entry_set_text(GTK_ENTRY(delimiter_entry), custom_delimiter);

  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(rename_columns_item), 
                              rename_columns);

  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(rename_worksheets_item), 
                              rename_worksheets);
}


void
sg_import_dialog ()
{
  GtkWidget *window = NULL;
  GtkWidget *frame;
  GtkWidget *main_box;
  GtkWidget *ok_button, *cancel_button;
  GtkWidget *action_area;
  GtkWidget *table;
  GtkAdjustment *adj;
  GtkRequisition req;

  window=gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
  gtk_window_set_title (GTK_WINDOW(window),"Arrange layers");

  gtk_window_set_modal (GTK_WINDOW(window),TRUE);

  /* Create widgets */
  main_box = gtk_vbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 10);
  gtk_container_add (GTK_CONTAINER (window), main_box);

  frame = gtk_frame_new("File Structure");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  table = gtk_table_new(2, 3, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 10);
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  delimiter_item = gtk_check_item_new_with_label("Delimiter"); 
  delimiter_combo = gtk_combo_new();
  delimiter_entry = gtk_entry_new();   

  gtk_widget_size_request(delimiter_entry, &req);
  gtk_widget_set_usize(delimiter_entry, 20, req.height);
  gtk_widget_set_usize(delimiter_combo, req.width/2, req.height);

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(delimiter_combo)->entry), FALSE);

  gtk_table_attach_defaults(GTK_TABLE(table), delimiter_item, 0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), delimiter_combo, 1, 2, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), delimiter_entry, 2, 3, 0, 1);

  frame = gtk_frame_new("File Header");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  table = gtk_table_new(1, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 10);
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 100., 1., 0., 0.); 
  skip_spin = gtk_spin_button_new(adj, 0, 0);   
  gtk_widget_set_usize(skip_spin, 40, req.height);

  gtk_entry_set_editable(GTK_ENTRY(skip_spin), FALSE);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new("Skip number of lines"),
                            0, 1, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            skip_spin,
                            1, 2, 0, 1);

  frame = gtk_frame_new("Options");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  table = gtk_table_new(2, 1, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 10);
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  rename_columns_item = gtk_check_item_new_with_label(
                        "Rename columns if 1st. line contains labels");
  rename_worksheets_item = gtk_check_item_new_with_label(
                        "Rename worksheet to data file name");

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            rename_columns_item,
                            0, 1, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            rename_worksheets_item,
                            0, 1, 1, 2);

  /* Action Area */
  action_area = gtk_hbutton_box_new ();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(action_area), 5);
  gtk_box_pack_end (GTK_BOX (main_box), action_area, FALSE, FALSE, 0);
  gtk_widget_show (action_area);

  ok_button = gtk_button_new_with_label ("Update Options");
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (action_area), ok_button, TRUE, TRUE, 0);
  gtk_widget_grab_default (ok_button);
  gtk_widget_show (ok_button);

  cancel_button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (action_area), cancel_button, TRUE, TRUE, 0);
  gtk_widget_show (cancel_button);

  /* connect signals */
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_destroy),
                             GTK_OBJECT (window));

  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
                             GTK_SIGNAL_FUNC (update_options),
                             GTK_OBJECT (window));

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
                      GTK_SIGNAL_FUNC (mw_destroy),NULL);

  /* Show widgets */

  init_dialog();

  gtk_widget_show_all (window);

  /* wait until dialog get destroyed */
  gtk_main();

}
