/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include "sg_text_dialog.h"

static char *popup_items[]={
                "Open",
                "Rename",
                "Remove",
};

static void sg_menu_exit				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_project				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_plot				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_worksheet			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_open				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_save				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_save_as				(GtkWidget *widget, 
							 gpointer data);

void
sg_worksheet_popup_show(GtkIconListItem *item, GdkEventButton *event)
{
    active_icon = item;

    gtk_menu_popup(GTK_MENU(worksheet_popup), NULL, NULL, NULL, NULL,
                   event->button, event->time);
}

void
sg_plot_popup_show(GtkIconListItem *item, GdkEventButton *event)
{
    active_icon = item;

    gtk_menu_popup(GTK_MENU(plot_popup), NULL, NULL, NULL, NULL,
                   event->button, event->time);
}


static void
plot_popup_activated(GtkWidget *widget, gpointer data)
{
 gchar *item;
 GtkIconList *iconlist;
 SGplot *plot;

 item = (gchar *)data;
 iconlist = GTK_ICON_LIST(iconlist2);

 plot =  (SGplot *) gtk_icon_list_get_link(active_icon);

 if(strcmp(item,"Open")==0)
   sg_plot_open(plot);

 if(strcmp(item,"Remove")==0){
   gint veto;
   gchar text[100]; 
   
   sprintf(text,"Remove %s ?", plot->name);

   veto = sg_accept_dialog(text);
   if(!veto) return;

   sg_project_remove_plot(plot);
 }

 if(strcmp(item,"Rename")==0)
   gtk_icon_list_set_active_icon(iconlist, active_icon);

}

static void
worksheet_popup_activated(GtkWidget *widget, gpointer data)
{
 gchar *item;
 GtkIconList *iconlist;
 SGworksheet *worksheet;

 item = (gchar *)data;
 iconlist = GTK_ICON_LIST(iconlist1);

 worksheet = (SGworksheet *) gtk_icon_list_get_link(active_icon);

 if(strcmp(item,"Open")==0)
   sg_worksheet_open(worksheet);

 if(strcmp(item,"Remove")==0){
   gint veto;
   gchar text[100];

   sprintf(text,"Remove %s ?", worksheet->name);

   veto = sg_accept_dialog(text);
   if(!veto) return;

   sg_project_remove_worksheet(worksheet);
 }

 if(strcmp(item,"Rename")==0)
   gtk_icon_list_set_active_icon(iconlist, active_icon);

}


void
sg_project_build_popups()
{
        GtkWidget *item;
        int i;

        worksheet_popup = gtk_menu_new();

        for (i=0; i < (sizeof(popup_items)/sizeof(popup_items[0])) ; i++){
                item=gtk_menu_item_new_with_label(popup_items[i]);
                gtk_signal_connect(GTK_OBJECT(item),"activate",
                                   (GtkSignalFunc) worksheet_popup_activated,
                                   popup_items[i]);
                gtk_widget_show(item);
                gtk_menu_append(GTK_MENU(worksheet_popup),item);
        }

        plot_popup = gtk_menu_new();

        for (i=0; i < (sizeof(popup_items)/sizeof(popup_items[0])) ; i++){
                item=gtk_menu_item_new_with_label(popup_items[i]);
                gtk_signal_connect(GTK_OBJECT(item),"activate",
                                   (GtkSignalFunc) plot_popup_activated,
                                   popup_items[i]);
                gtk_widget_show(item);
                gtk_menu_append(GTK_MENU(plot_popup),item);
        }

}

void
sg_project_build_menu()
{
 menu_bar = gtk_menu_bar_new();

 file_menu = gtk_menu_new();

 menu_item = gtk_menu_item_new_with_label("Project");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_project, NULL);

 menu_item = gtk_menu_item_new_with_label("Worksheet");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_worksheet, NULL);

 menu_item = gtk_menu_item_new_with_label("Plot");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_plot, NULL);

 menu = gtk_menu_new();

 menu_item = gtk_menu_item_new_with_label("New");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);

 gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),file_menu);

 menu_item = gtk_menu_item_new_with_label("Open");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_open, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Save");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_save, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Save As");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_save_as, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Exit");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_exit, NULL);
 
 root_menu = gtk_menu_item_new_with_label("File");
 gtk_menu_item_set_submenu (GTK_MENU_ITEM(root_menu), menu);
 gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);
 gtk_widget_show(root_menu);

 menu = gtk_menu_new();

/*
 menu_item = gtk_menu_item_new_with_label("Contents");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
*/

 menu_item = gtk_menu_item_new_with_label("About SciGraphica");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_about_dialog, NULL);

 root_menu = gtk_menu_item_new_with_label("Help");
 gtk_menu_item_right_justify(GTK_MENU_ITEM(root_menu));
 gtk_menu_item_set_submenu (GTK_MENU_ITEM(root_menu), menu);
 gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);
 gtk_widget_show(root_menu);

}


static void
sg_menu_exit(GtkWidget *widget, gpointer data)
{
  gint veto;

  sg_dialogs_destroy();

  veto = sg_accept_dialog("Exit SciGraphica?");
  if(!veto) return;

  veto = sg_accept_dialog("Save project before leaving?");
  if(veto) sg_menu_save(widget, data);

  sg_project_close();

  gtk_main_quit();
}

static void
sg_menu_new_plot(GtkWidget *widget, gpointer data)
{
  sg_project_new_plot();
}

static void
sg_menu_new_worksheet(GtkWidget *widget, gpointer data)
{
  sg_project_new_worksheet();
}

static void
sg_menu_new_project(GtkWidget *widget, gpointer data)
{
  gint veto;

  veto = sg_accept_dialog("Quit this project?");
  if(!veto) return;
  
  if(num_worksheets > 0 || num_plots > 0){ 
    veto = sg_accept_dialog("Save project before leaving?");
    if(veto) sg_menu_save(widget, data);
  }

  sg_project_close();
  sg_project_new();
  sg_project_new_worksheet();
}

static void
sg_menu_open(GtkWidget *widget, gpointer data)
{
  gboolean veto;

  veto = sg_accept_dialog("Save project before leaving?");
  if(veto)
     sg_project_export();

  sg_project_close();
  sg_project_new();
  sg_project_import();
}

static void
sg_menu_save(GtkWidget *widget, gpointer data)
{
/*  sg_message_dialog("Sorry. This feature is not implemented", 2);
*/
  sg_project_export();
}

static void
sg_menu_save_as(GtkWidget *widget, gpointer data)
{
/*  sg_message_dialog("Sorry. This feature is not implemented", 2);
*/
  sg_project_export();
}
