/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"


static GtkWidget *dataset_combo;
static GtkWidget *symbol_type_combo;
static GtkWidget *symbol_style_combo;
static GtkWidget *line_style_combo;
static GtkWidget *connector_combo;
static GtkWidget *symbol_color_combo;
static GtkWidget *line_color_combo;
static GtkWidget *line_width_spin;
static GtkWidget *err_width_spin;
static GtkWidget *err_caps_spin;
static GtkWidget *symbol_size_spin;
static GtkWidget *symbol_width_spin;
static GtkWidget *yerr_check;
static GtkWidget *xerr_check;
static GtkWidget *fill_check;
static GtkWidget *text_entry;
static GtkWidget *xline_combo;
static GtkWidget *yline_combo;
static GtkWidget *xline_color_combo;
static GtkWidget *xline_width_spin;
static GtkWidget *font_combo;
static GtkWidget *font_color_combo;
static GtkWidget *labels_check;
GList *columns;

static gchar *symbol_types[] = {"None",
                               "Square",
                               "Circle",
                               "Up triangle",
                               "Down triangle",
                               "Diamond",
                               "Plus",
                               "Cross",
                               "Star",
                               "Impulse",
                               NULL
                               };

static gchar *symbol_styles[] = {"Empty",
                                "Filled",
                                "Opaque",
                                NULL
                               };

static gchar *line_styles[] = {"None",
                              "Solid",
                              "Dotted",
                              "Dashed",
                              "Dot-dash",
                              "Dot-dot-dash",
                              "Dot-dash-dash",
                              NULL
                             };

static gchar *connectors[] = {"None",
                             "Straight",
                             "Spline",
                             "H-V step",
                             "V-H step",
                             "Middle step",
                             NULL
                             };


void
sg_style_dialog_update_plot(GtkWidget *widget)
{
  SGplot *plot;
  GtkWidget *child;
  GdkColormap *colormap;
  GdkColor line_color, symbol_color, font_color;
  gint line_style, line_width;
  gint symbol_style, symbol_type, symbol_size, symbol_width, connector;
  gint row, col;
  gint font_height;
  gchar *name;
  gchar *legend_text;
  gchar *font_name;
 
  colormap = gdk_colormap_get_system(); 

  child = GTK_LIST(GTK_COMBO(line_style_combo)->list)->selection->data;
  line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(line_style_combo)->list), child);

  child = GTK_LIST(GTK_COMBO(connector_combo)->list)->selection->data;
  connector = gtk_list_child_position(GTK_LIST(GTK_COMBO(connector_combo)->list), child);

  line_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(line_width_spin));

  child = GTK_LIST(GTK_COMBO(symbol_style_combo)->list)->selection->data;
  symbol_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(symbol_style_combo)->list), child);

  child = GTK_LIST(GTK_COMBO(symbol_type_combo)->list)->selection->data;
  symbol_type = gtk_list_child_position(GTK_LIST(GTK_COMBO(symbol_type_combo)->list), child);

  symbol_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(symbol_size_spin));
  symbol_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(symbol_width_spin));

  row = GTK_COLOR_COMBO(line_color_combo)->row;
  col = GTK_COLOR_COMBO(line_color_combo)->column;
  name = gtk_color_combo_get_color_at(GTK_COLOR_COMBO(line_color_combo), row, col);
  gdk_color_parse(name, &line_color);
  gdk_color_alloc(colormap, &line_color);

  row = GTK_COLOR_COMBO(symbol_color_combo)->row;
  col = GTK_COLOR_COMBO(symbol_color_combo)->column;
  name = gtk_color_combo_get_color_at(GTK_COLOR_COMBO(symbol_color_combo), row, col);
  gdk_color_parse(name, &symbol_color);
  gdk_color_alloc(colormap, &symbol_color);

  gtk_plot_data_set_symbol(active_dataset->real_data,
                              (GtkPlotSymbolType) symbol_type,
                              (GtkPlotSymbolStyle) symbol_style,
                              symbol_size,
                              symbol_width,
                              &symbol_color);

  gtk_plot_data_set_connector(active_dataset->real_data, connector);

  gtk_plot_data_set_line_attributes(active_dataset->real_data,
                                       (GtkPlotLineStyle) line_style,
                                       line_width,
                                       &line_color);
 
  child = GTK_LIST(GTK_COMBO(xline_combo)->list)->selection->data;
  line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(xline_combo)->list), child);

  line_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(xline_width_spin));

  row = GTK_COLOR_COMBO(xline_color_combo)->row;
  col = GTK_COLOR_COMBO(xline_color_combo)->column;
  name = gtk_color_combo_get_color_at(GTK_COLOR_COMBO(xline_color_combo), row, col);
  gdk_color_parse(name, &line_color);
  gdk_color_alloc(colormap, &line_color);

  gtk_plot_data_set_x_attributes(active_dataset->real_data, 
                                    line_style,
                                    line_width,
                                    &line_color);

  child = GTK_LIST(GTK_COMBO(yline_combo)->list)->selection->data;
  line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(yline_combo)->list), child);

  gtk_plot_data_set_y_attributes(active_dataset->real_data, 
                                    line_style,
                                    line_width,
                                    &line_color);


  active_dataset->real_data->show_xerrbars = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(xerr_check));
  active_dataset->real_data->show_yerrbars = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(yerr_check));

  active_dataset->real_data->xerrbar_caps = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(err_caps_spin));
  active_dataset->real_data->yerrbar_caps = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(err_caps_spin));
                                        
  active_dataset->real_data->xerrbar_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(err_width_spin));
  active_dataset->real_data->yerrbar_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(err_width_spin));

  legend_text = gtk_entry_get_text(GTK_ENTRY(text_entry));

  gtk_plot_data_set_legend(active_dataset->real_data, legend_text);

  active_dataset->real_data->show_labels = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(labels_check));

  font_name = GTK_FONT_COMBO(font_combo)->psfont->psname;
  font_height = GTK_FONT_COMBO(font_combo)->height;
  
  row = GTK_COLOR_COMBO(xline_color_combo)->row;
  col = GTK_COLOR_COMBO(xline_color_combo)->column;
  name = gtk_color_combo_get_color_at(GTK_COLOR_COMBO(font_color_combo), row, col);
  gdk_color_parse(name, &font_color);
  gdk_color_alloc(colormap, &font_color);

  gtk_plot_data_labels_set_attributes(active_dataset->real_data,
                                      font_name, font_height, 90, 
                                      &font_color, NULL);


  plot = (SGplot *)active_layer->parent;
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}


static void
init_dialog(GtkEntry *entry)
{
  GList *list;
  GdkColor line_color, symbol_color;
  GtkPlotLineStyle line_style;
  gint line_width;
  GtkPlotSymbolStyle symbol_style;
  GtkPlotSymbolType symbol_type; 
  gint symbol_width, symbol_size, connector;
  gchar *text;
  GtkPSFont *font;
  SGdataset *dataset = NULL;

  text = gtk_entry_get_text(entry);

  list = active_layer->datasets;
  while(list){
    dataset = (SGdataset *)list->data;
    if(strcmp(text, dataset->real_data->name) == 0) break;
    list = list->next;
  }

  active_dataset = dataset;

  gtk_plot_data_get_line_attributes(active_dataset->real_data,
                                       &line_style, &line_width, &line_color);
 
  gtk_plot_data_get_symbol(active_dataset->real_data,
                              &symbol_type, &symbol_style, 
                              &symbol_size, &symbol_width, &symbol_color);

  connector = gtk_plot_data_get_connector(active_dataset->real_data);

  gtk_list_select_item(GTK_LIST(GTK_COMBO(line_style_combo)->list), line_style); 
  gtk_list_select_item(GTK_LIST(GTK_COMBO(connector_combo)->list), connector); 

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(line_width_spin), line_width);

  gtk_list_select_item(GTK_LIST(GTK_COMBO(symbol_type_combo)->list), symbol_type); 
  gtk_list_select_item(GTK_LIST(GTK_COMBO(symbol_style_combo)->list), symbol_style); 

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(symbol_size_spin), symbol_size);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(symbol_width_spin), symbol_width);

  sg_color_combo_init(GTK_COLOR_COMBO(line_color_combo), line_color);
  sg_color_combo_init(GTK_COLOR_COMBO(symbol_color_combo), symbol_color);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(xerr_check), 
                               dataset->real_data->show_xerrbars);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(yerr_check), 
                               dataset->real_data->show_yerrbars);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(err_width_spin),
                            dataset->real_data->xerrbar_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(err_caps_spin),
                            dataset->real_data->xerrbar_caps);

  if(dataset->real_data->legend)
    gtk_entry_set_text(GTK_ENTRY(text_entry), dataset->real_data->legend);

  gtk_list_select_item(GTK_LIST(GTK_COMBO(xline_combo)->list), 
                       dataset->real_data->x_line.line_style); 
  gtk_list_select_item(GTK_LIST(GTK_COMBO(xline_combo)->list), 
                       dataset->real_data->y_line.line_style); 
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(xline_width_spin),
                            dataset->real_data->x_line.line_width);
  sg_color_combo_init(GTK_COLOR_COMBO(xline_color_combo), 
                      dataset->real_data->x_line.color);

  font = gtk_psfont_get_font(dataset->real_data->labels_attr.font);
  gtk_font_combo_select(GTK_FONT_COMBO(font_combo),
                        font->family,
                        font->bold,
                        font->italic,
                        dataset->real_data->labels_attr.height);

  sg_color_combo_init(GTK_COLOR_COMBO(font_color_combo),
                      dataset->real_data->labels_attr.fg);
}


GtkWidget *
sg_style_dialog_new (SGlayer *layer)
{
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *main_box;
  GtkWidget *table, *table1;
  GList *list;
  GtkWidget *item;
  SGdataset *dataset;
  GtkRequisition req;
  GtkAdjustment *adj;

  active_layer = layer;

  /* Create widgets */
  main_box = gtk_hbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 5);

  table = gtk_table_new(3, 4, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
  gtk_table_set_row_spacings(GTK_TABLE(table), 10);
  gtk_box_pack_start(GTK_BOX(main_box), table, FALSE, FALSE, 0);

  label = gtk_label_new("Select Dataset:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);

  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

  dataset_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table),dataset_combo,2,3,0,1);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dataset_combo)->entry), FALSE);
  gtk_widget_size_request(dataset_combo, &req);
  req.width /= 2;

  list = layer->datasets;
  while(list){
   dataset = (SGdataset *)list->data;
   item = gtk_list_item_new_with_label(dataset->real_data->name);
   gtk_widget_show(item);
   gtk_container_add(GTK_CONTAINER(GTK_COMBO(dataset_combo)->list), item);
   list = list->next;
  }

  /* Lines */

  frame = gtk_frame_new("Line");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table),frame,0,1,1,2);

  table1 = gtk_table_new(6, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table1), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table1), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table1), 5);
  gtk_container_add (GTK_CONTAINER (frame), table1);

  label = gtk_label_new("Connect:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,0,1);

  connector_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),connector_combo,1,2,0,1);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(connector_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(connector_combo), connectors);

  label = gtk_label_new("Type:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,1,2);

  line_style_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),line_style_combo,1,2,1,2);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(line_style_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(line_style_combo), line_styles);

  label = gtk_label_new("Width:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,2,3);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  line_width_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(line_width_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(line_width_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),line_width_spin,1,2,2,3);

  label = gtk_label_new("Color:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,3,4);

  line_color_combo = gtk_color_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),line_color_combo,1,2,3,4);

  fill_check = gtk_check_item_new_with_label("Fill area");
  gtk_table_attach_defaults(GTK_TABLE(table1),fill_check,0,2,4,5);

  gtk_widget_set_usize(connector_combo, req.width, req.height);
  gtk_widget_set_usize(line_width_spin, req.width, req.height);
  gtk_widget_set_usize(line_style_combo, req.width, req.height);

  /* Symbols */

  frame = gtk_frame_new("Symbol");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table),frame,1,2,1,2);

  table1 = gtk_table_new(6, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table1), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table1), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table1), 5);
  gtk_container_add (GTK_CONTAINER (frame), table1);

  label = gtk_label_new("Shape:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,0,1);

  symbol_type_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),symbol_type_combo,1,2,0,1);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(symbol_type_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(symbol_type_combo), symbol_types);

  label = gtk_label_new("Style:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,1,2);

  symbol_style_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),symbol_style_combo,1,2,1,2);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(symbol_style_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(symbol_style_combo), symbol_styles);

  label = gtk_label_new("Size:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,2,3);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 20., 1., 1., 0.);
  symbol_size_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(symbol_size_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(symbol_size_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),symbol_size_spin,1,2,2,3);

  label = gtk_label_new("Thickness:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,3,4);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  symbol_width_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(symbol_width_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(symbol_width_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),symbol_width_spin,1,2,3,4);

  label = gtk_label_new("Color:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,4,5);

  symbol_color_combo = gtk_color_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),symbol_color_combo,1,2,4,5);

  gtk_widget_set_usize(symbol_type_combo, req.width, req.height);
  gtk_widget_set_usize(symbol_size_spin, req.width, req.height);
  gtk_widget_set_usize(symbol_width_spin, req.width, req.height);
  gtk_widget_set_usize(symbol_style_combo, req.width, req.height);

  /* X-Y lines */

  frame = gtk_frame_new("X-Y lines");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table),frame,2,3,1,2);

  table1 = gtk_table_new(6, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table1), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table1), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table1), 5);
  gtk_container_add (GTK_CONTAINER (frame), table1);

  label = gtk_label_new("X line:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,0,1);

  xline_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),xline_combo,1,2,0,1);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(xline_combo)->entry), FALSE);
  sg_combo_set_items(GTK_COMBO(xline_combo), line_styles);

  label = gtk_label_new("Y line:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,1,2);

  yline_combo = gtk_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),yline_combo,1,2,1,2);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(yline_combo)->entry), FALSE);
  sg_combo_set_items(GTK_COMBO(yline_combo), line_styles);

  label = gtk_label_new("Width:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,2,3);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  xline_width_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(xline_width_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(xline_width_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),xline_width_spin,1,2,2,3);

  label = gtk_label_new("Color:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,3,4);

  xline_color_combo = gtk_color_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table1),xline_color_combo,1,2,3,4);

  gtk_widget_set_usize(xline_width_spin, req.width, req.height);
  gtk_widget_set_usize(xline_combo, req.width, req.height);
  gtk_widget_set_usize(yline_combo, req.width, req.height);

  /* Error bars */

  frame = gtk_frame_new("Error bars");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table),frame,3,4,1,2);

  table1 = gtk_table_new(6, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table1), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table1), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table1), 5);
  gtk_container_add (GTK_CONTAINER (frame), table1);

  label = gtk_label_new("Width:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,0,1);

  label = gtk_label_new("Caps:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table1),label,0,1,1,2);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  err_width_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(err_width_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(err_width_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),err_width_spin,1,2,0,1);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 20., 1., 1., 0.);
  err_caps_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(err_caps_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(err_caps_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table1),err_caps_spin,1,2,1,2);

  xerr_check = gtk_check_item_new_with_label("show xerrbars");
  yerr_check = gtk_check_item_new_with_label("show yerrbars");

  gtk_widget_set_usize(err_width_spin, req.width, req.height);
  gtk_widget_set_usize(err_caps_spin, req.width, req.height);

  gtk_table_attach_defaults(GTK_TABLE(table1),xerr_check,0,3,2,3);
  gtk_table_attach_defaults(GTK_TABLE(table1),yerr_check,0,3,3,4);

  /* Labels */

  frame = gtk_frame_new("Labels");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 4, 2, 3);

  font_combo = gtk_font_combo_new();
  gtk_container_add(GTK_CONTAINER(frame), font_combo);
  gtk_container_set_border_width(GTK_CONTAINER(font_combo), 5);
  gtk_toolbar_append_space(GTK_TOOLBAR(font_combo));

  font_color_combo = gtk_color_combo_new();
  gtk_container_add(GTK_CONTAINER(font_combo), font_color_combo);

  gtk_toolbar_append_space(GTK_TOOLBAR(font_combo));
  gtk_toolbar_append_space(GTK_TOOLBAR(font_combo));
  gtk_toolbar_append_space(GTK_TOOLBAR(font_combo));

  labels_check = gtk_check_item_new_with_label("show labels");
  gtk_container_add(GTK_CONTAINER(font_combo), labels_check); 

  /* Legends */

  frame = gtk_frame_new("Legend");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 4, 3, 4);

  text_entry = gtk_entry_new();
  gtk_container_add(GTK_CONTAINER(frame), text_entry);

  /* connect signals */
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dataset_combo)->entry),
                     "changed",
                     (GtkSignalFunc) init_dialog, NULL);

  gtk_signal_connect(GTK_OBJECT(symbol_color_combo),"changed",
                     GTK_SIGNAL_FUNC(sg_color_combo_changed), NULL);

  gtk_signal_connect(GTK_OBJECT(line_color_combo),"changed",
                     GTK_SIGNAL_FUNC(sg_color_combo_changed), NULL);

  gtk_signal_connect(GTK_OBJECT(xline_color_combo),"changed",
                     GTK_SIGNAL_FUNC(sg_color_combo_changed), NULL);

  gtk_signal_connect(GTK_OBJECT(font_color_combo),"changed",
                     GTK_SIGNAL_FUNC(sg_color_combo_changed), NULL);

/******************************************************************/
  active_dataset = (SGdataset *)layer->datasets->data;

  init_dialog(GTK_ENTRY(GTK_COMBO(dataset_combo)->entry)); 

/******************************************************************/

  return main_box;
}

