/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkframe.h>
#include "sg_toolbox.h"
#include "sg_toolbox_icons.h"

#define NROWS 6
#define NCOLS 2

static void         sg_toolbox_class_init      (SGtoolboxClass *klass);
static void         sg_toolbox_init            (SGtoolbox      *toolbox);
static void         select_tool                (GtkWidget *widget, 
                                                gpointer data);

static GtkTableClass *parent_class = NULL;

static void
sg_toolbox_class_init (SGtoolboxClass * klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  parent_class = gtk_type_class (gtk_table_get_type ());
  object_class = (GtkObjectClass *) klass;
  widget_class = (GtkWidgetClass *) klass;

}

static void
select_tool(GtkWidget *widget, gpointer data)
{
  gint row, column;
  GdkGC *xor_gc;
  GdkGCValues values;
  GtkWidget *tool;
  SGtoolbox *toolbox;
 
  toolbox = SG_TOOLBOX(data);

  gdk_gc_get_values(widget->style->fg_gc[GTK_STATE_SELECTED], &values);
  values.function = GDK_INVERT;
  values.foreground = widget->style->white;
  values.subwindow_mode = GDK_INCLUDE_INFERIORS;
  xor_gc = gdk_gc_new_with_values(widget->window,
                                  &values,
                                  GDK_GC_FOREGROUND |
                                  GDK_GC_FUNCTION |
                                  GDK_GC_SUBWINDOW);

  gdk_gc_set_foreground(xor_gc, &widget->style->bg[GTK_STATE_SELECTED]);

  for(row = 0; row < NROWS-4; row++){
    for(column = 0; column < NCOLS; column++){
       tool = toolbox->button[row][column];
       if(tool != widget){
          if(GTK_TOGGLE_BUTTON(tool)->active == TRUE)
              gdk_draw_rectangle(GTK_PIXMAP(GTK_BIN(tool)->child)->pixmap,
                                 xor_gc,
                                 TRUE, 2, 2, 28, 28);
          GTK_BUTTON(tool)->button_down=FALSE;
          GTK_TOGGLE_BUTTON(tool)->active=FALSE;
          gtk_widget_set_state(tool, GTK_STATE_NORMAL);
       }else{
          if(GTK_TOGGLE_BUTTON(tool)->active != TRUE)
              gdk_draw_rectangle(GTK_PIXMAP(GTK_BIN(tool)->child)->pixmap,
                                 xor_gc,
                                 TRUE, 2, 2, 28, 28);
          GTK_BUTTON(tool)->button_down=TRUE;
          GTK_TOGGLE_BUTTON(tool)->active=TRUE;
          gtk_widget_set_state(tool, GTK_STATE_ACTIVE);
       }
       if(GTK_TOGGLE_BUTTON(tool)->active == TRUE)
          gdk_draw_rectangle(GTK_PIXMAP(GTK_BIN(tool)->child)->pixmap,
                             xor_gc,
                             TRUE, 2, 2, 28, 28);
       gtk_widget_queue_draw(tool);
    }
   }

}

static void
sg_toolbox_init (SGtoolbox *toolbox)
{
  GtkTable *table;
  GtkWidget *widget;
  gint i, j;
  GdkPixmap *tool;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GdkColormap *colormap;

  widget = GTK_WIDGET(toolbox);
  table = GTK_TABLE(widget);

  gtk_table_resize(table, 7, 2);
  table->homogeneous = FALSE;

  colormap = gdk_colormap_get_system();

  for(i = 0; i < NROWS; i++){
    for(j = 0; j < NCOLS; j++){

      if(i < NROWS-4){
        toolbox->button[i][j] = gtk_toggle_button_new();

        gtk_table_attach (GTK_TABLE(table),
                          toolbox->button[i][j],
                          j, j+1, i, i+1, GTK_SHRINK, GTK_SHRINK, 0, 0);

        gtk_signal_connect (GTK_OBJECT (toolbox->button[i][j]), "toggled",
                            (GtkSignalFunc) select_tool,
                            toolbox);
      }
      else
      {
        toolbox->button[i][j] = gtk_button_new();

        gtk_table_attach (GTK_TABLE(table),
                          toolbox->button[i][j],
                          j, j+1, i+1, i+2, GTK_SHRINK, GTK_SHRINK, 0, 0);
      }

      gtk_button_set_relief(GTK_BUTTON(toolbox->button[i][j]),
                            GTK_RELIEF_NONE);


      gtk_widget_set_usize(toolbox->button[i][j], 38, 38);
      gtk_widget_show(toolbox->button[i][j]);

    }
  }

  gtk_table_attach(table, gtk_hseparator_new(), 0, 2, 2, 3,
                   GTK_FILL, GTK_SHRINK, 5, 5);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             arrow);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[0][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             zoom);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[0][1]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             target);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[1][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             text);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[1][1]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             autoscale_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[2][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             fitpage_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[2][1]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             zoom_plus_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[3][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             zoom_minus_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[3][1]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             arrow_up_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[4][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             arrow_down_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[4][1]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             arrow_left_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[5][0]),pixmap);
  gtk_widget_show(pixmap);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             arrow_right_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  gtk_container_add(GTK_CONTAINER(toolbox->button[5][1]),pixmap);
  gtk_widget_show(pixmap);

}

guint
sg_toolbox_get_type ()
{
  static guint toolbox_type = 0;

  if (!toolbox_type)
    {
      GtkTypeInfo toolbox_info =
      {
	"SGtoolbox",
	sizeof (SGtoolbox),
	sizeof (SGtoolboxClass),
	(GtkClassInitFunc) sg_toolbox_class_init,
	(GtkObjectInitFunc) sg_toolbox_init,
	NULL,
	NULL,
	(GtkClassInitFunc) NULL,
      };
      toolbox_type = gtk_type_unique (gtk_table_get_type (), &toolbox_info);
    }
  return toolbox_type;
}

GtkWidget *
sg_toolbox_new ()
{
  SGtoolbox *toolbox;

  toolbox = gtk_type_new (sg_toolbox_get_type ());

  return(GTK_WIDGET(toolbox));
}



