/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SG_WORKSHEET_H__
#define __SG_WORKSHEET_H__

typedef struct _SGworksheet SGworksheet;
typedef struct _SGcolumn SGcolumn;
typedef struct _SGhiddencell SGhiddencell;

typedef enum
{
  SG_COLUMN_NUMBER,
  SG_COLUMN_TEXT,
  SG_COLUMN_DATE,
  SG_COLUMN_TIME,
} SGcolumnformat;

struct _SGcolumn
{
  SGcolumnformat format;

  gint precision;

  gchar *exp;
};

struct _SGhiddencell
{
  gchar *formula;
  gboolean updated;
};


struct _SGworksheet 
{
  gchar *name;
  gint last_column;
  gint x, y, width, height;

  gint begin, end;

  gboolean is_mapped;

  GtkWidget *sheet;
  GtkWidget *sw;
  GtkWidget *window;
  GtkWidget *label;
  GtkIconList *iconlist;
  GtkIconListItem *icon;

  SGcolumn *column;
};

SGworksheet 	*sg_worksheet_new		(GtkIconList *iconlist, 
						 gchar *name, 
                                                 gint nrows, gint ncols);

void		 sg_worksheet_open		(SGworksheet *worksheet); 
void		 sg_worksheet_hide		(SGworksheet *worksheet); 
gint 		 sg_worksheet_rename		(SGworksheet *worksheet,
						 gchar *name); 
SGworksheet     *sg_worksheet_get_by_name       (gchar *name);
void		 sg_worksheet_set_begin		(SGworksheet *worksheet,
						 gint row); 
void		 sg_worksheet_set_end		(SGworksheet *worksheet,
						 gint row); 
void		 sg_worksheet_reset		(SGworksheet *worksheet);
void		 sg_worksheet_remove		(SGworksheet *worksheet);
void		 sg_worksheet_add_columns	(SGworksheet *worksheet,
						 gint n);
void		 sg_worksheet_add_rows		(SGworksheet *worksheet,
						 gint n);
void		 sg_worksheet_insert_columns	(SGworksheet *worksheet,
						 gint col,
						 gint n);
void		 sg_worksheet_insert_rows	(SGworksheet *worksheet,
						 gint row,
						 gint n);
void		 sg_worksheet_delete_columns	(SGworksheet *worksheet,
						 gint col,
						 gint n);
void		 sg_worksheet_delete_rows	(SGworksheet *worksheet,
						 gint row,
						 gint n);
void 		 sg_worksheet_column_set_format	(SGworksheet *worksheet,
                               			 gint column, 
                               			 SGcolumnformat format,
                               			 gint precision);
void 		 sg_worksheet_column_set_exp	(SGworksheet *worksheet,
                               			 gint column, 
                               			 gchar *exp); 
void 		 sg_worksheet_cell_set		(SGworksheet *worksheet,
						 gint row, gint col, 
						 gchar *text,
                                                 gboolean eval);
char            *sg_worksheet_cell_get          (SGworksheet *worksheet,
                                                 gint row, gint col);





#endif /* __SG_WORKSHEET_H__ */
